@@ -14,36 +14,19 @@
 package Win32::API::Callback;
 use strict;
 use warnings;
-use vars qw( $VERSION @ISA $Stage2FuncPtrPkd );
+use vars qw( $VERSION $Stage2FuncPtrPkd );
 
-$VERSION = '0.79';
+$VERSION = '0.81';
 
-
-require Exporter;      # to export the constants to the main:: space
-require DynaLoader;    # to dynuhlode the module.
-@ISA = qw( Exporter DynaLoader );
+#require XSLoader;    # to dynuhlode the module. #already loaded by Win32::API
 #use Data::Dumper;
 
-sub DEBUG {
-    if ($WIN32::API::DEBUG) {
-        printf @_ if @_ or return 1;
-    }
-    else {
-        return 0;
-    }
-}
-
 use Win32::API qw ( WriteMemory ) ;
-use Win32::API::Type;
-use Config;
-#use Win32::API::Struct; #not implemented
-
 
 BEGIN {
     #there is supposed to be 64 bit IVs on 32 bit perl compatibility here
     #but it is untested
-    #Win64 added in 5.7.3
-    eval "sub IVSIZE () { ".length(pack($] >= 5.007003 ? 'J' : 'I' ,0))." }";
+    *IVSIZE = *Win32::API::IVSIZE;
     #what kind of stack processing/calling convention/machine code we needed
     eval "sub ISX64 () { ".(Win32::API::PTRSIZE() == 8 ?  1 : 0)." }";
     eval 'sub OPV () {'.$].'}';
@@ -58,11 +41,13 @@ BEGIN {
     if(OPV <= 5.008000){ #don't have unpackstring in C
         eval('sub _CallUnpack {return unpack($_[0], $_[1]);}');
     }
+    *DEBUGCONST = *Win32::API::DEBUGCONST;
+    *DEBUG = *Win32::API::DEBUG;
 }
 #######################################################################
 # dynamically load in the API extension module.
 #
-bootstrap Win32::API::Callback;
+XSLoader::load 'Win32::API::Callback', $VERSION;
 
 #######################################################################
 # PUBLIC METHODS
@@ -100,10 +85,10 @@ sub new {
     $self->{sub} = $proc;
     $self->{cdecl} = Win32::API::calltype_to_num($callconvention);
 
-    DEBUG "(PM)Callback::new: calling CallbackCreate($self)...\n";
+    DEBUG "(PM)Callback::new: calling CallbackCreate($self)...\n" if DEBUGCONST;
     my $hproc = MakeCB($self);
 
-    DEBUG "(PM)Callback::new: hproc=$hproc\n";
+    DEBUG "(PM)Callback::new: hproc=$hproc\n" if DEBUGCONST;
 
     $self->{code} = $hproc;
 
@@ -113,7 +98,7 @@ sub new {
 
 sub MakeStruct {
     my ($self, $n, $addr) = @_;
-    DEBUG "(PM)Win32::API::Callback::MakeStruct: got self='$self'\n";
+    DEBUG "(PM)Win32::API::Callback::MakeStruct: got self='$self'\n" if DEBUGCONST;
     my $struct = Win32::API::Struct->new($self->{intypes}->[$n]);
     $struct->FromMemory($addr);
     return $struct;
